// Strings.cpp
#include  <windows.h>
#include  <string.h>
#include  "stdio.h"
#include  "Strings.h"

#ifdef NUMERIC_CONVERSIONS 
   extern HINSTANCE hLib;
   extern int (*pSPrtf)(char*, const char*, ... );
   #ifdef x64
      extern ssize_t (*pAbs64)(ssize_t);
      extern ssize_t (*pAtoi64)(const char*);
   #else
      extern ssize_t (*pAbs)(ssize_t);
   #endif
#else
   #ifdef FORMAT_MONEY
      extern HINSTANCE hLib;
      extern int (*pSPrtf)(char*, const char*, ... );
      #ifdef x64
         extern ssize_t (*pAbs64)(ssize_t);
         extern ssize_t (*pAtoi64)(const char*);
      #else
         extern ssize_t (*pAbs)(ssize_t);
      #endif
   #endif
#endif


String operator+(char* lhs, String& rhs)              //global function
{
 String sr=lhs;
 sr=sr+rhs;

 return sr;
}


String::String()                                      // Uninitialized Constructor
{
 lpBuffer=(char*)operator new(MINIMUM_ALLOCATION);
 this->lpBuffer=new char[MINIMUM_ALLOCATION];
 lpBuffer[0]='\0';
 this->iCapacity=MINIMUM_ALLOCATION-1;
 this->iLen=0;
}


String::String(const int iSize, bool blnFillNulls)  //Constructor Creates String With Custom Sized
{                                                   //Buffer (rounded up to paragraph boundary)
 int iNewSize=(iSize/16+1)*16;
 this->lpBuffer=new char[iNewSize];
 this->iCapacity=iNewSize-1;
 this->iLen=0;
 this->lpBuffer[0]='\0';
 if(blnFillNulls)
 {
    for(size_t i=0; i<this->iCapacity; i++)
        this->lpBuffer[i]=0;
 }
}


String::String(const char ch)  //Constructor: Initializes with char
{
 this->iLen=1;
 int iNewSize=MINIMUM_ALLOCATION;
 this->lpBuffer=new char[iNewSize];
 this->iCapacity=iNewSize-1;
 this->lpBuffer[0]=ch, this->lpBuffer[1]='\0';
}


String::String(const char* pStr)  //Constructor: Initializes with char*
{
 this->iLen=(int)strlen(pStr);
 int iNewSize=(this->iLen/16+1)*16;
 this->lpBuffer=new char[iNewSize];
 this->iCapacity=iNewSize-1;
 strcpy(lpBuffer,pStr);
}


String::String(const String& s)  //Constructor Initializes With Another String, i.e., Copy Constructor
{
 int iNewSize=(s.iLen/16+1)*16;
 this->iLen=s.iLen;
 this->lpBuffer=new char[iNewSize];
 this->iCapacity=iNewSize-1;
 strcpy(this->lpBuffer,s.lpBuffer);
}


#ifdef NUMERIC_CONVERSIONS
   #ifdef x64
      String::String(size_t iNum)
      {
       this->lpBuffer=new char[32];
       this->iCapacity=31;
       this->iLen=sprintf(this->lpBuffer,"%Iu",iNum);
      }


      String::String(ssize_t iNum)
      {
       this->lpBuffer=new char[32];
       this->iCapacity=31;
       this->iLen=sprintf(this->lpBuffer,"%Id",iNum);
      }
   #endif


   String::String(int iNum)
   {
    this->lpBuffer=new char[32];
    this->iCapacity=15;
    this->iLen=sprintf(this->lpBuffer,"%d",iNum);
   }


   String::String(unsigned int iNum)
   {
    this->lpBuffer=new char[16];
    this->iCapacity=15;
    this->iLen=sprintf(this->lpBuffer,"%u",iNum);
   }


   String::String(double dblNumber)
   {
    char szBuffer[48];

    this->iLen=pSPrtf(szBuffer,"%16.8f",dblNumber);
    int iNewSize=(this->iLen/16+1)*16;
    this->lpBuffer=new char[iNewSize];
    this->iCapacity=iNewSize-1;
    strcpy(this->lpBuffer,szBuffer);
   }
#endif


String& String::operator=(const char ch)
{
 this->lpBuffer[0]=ch, this->lpBuffer[1]='\0';
 this->iLen=1;
 return *this;
}


String& String::operator=(const char* pStr)  // Assign TCHAR* to String
{
 size_t iNewLen=strlen(pStr);
 if(iNewLen>this->iCapacity)
 {
    delete [] this->lpBuffer;
    int iNewSize=(iNewLen*EXPANSION_FACTOR/16+1)*16;
    this->lpBuffer=new char[iNewSize];
    this->iCapacity=iNewSize-1;
 }
 strcpy(this->lpBuffer,pStr);
 this->iLen=iNewLen;

 return *this;
}


String& String::operator=(const String& strAnother)
{
 if(this==&strAnother)
    return *this;
 if(strAnother.iLen>this->iCapacity)
 {
    delete [] this->lpBuffer;
    int iNewSize=(strAnother.iLen*EXPANSION_FACTOR/16+1)*16;
    this->lpBuffer=new char[iNewSize];
    this->iCapacity=iNewSize-1;
 }
 strcpy(this->lpBuffer,strAnother.lpBuffer);
 this->iLen=strAnother.iLen;

 return *this;
}


#ifdef NUMERIC_CONVERSIONS
   #ifdef x64
      String& String::operator=(size_t iNum)
      {
       if(this->iCapacity>=32)
          this->iLen=sprintf(this->lpBuffer,"%Iu",iNum);
       else
       {
          delete [] this->lpBuffer;
          this->lpBuffer=new char[32];
          this->iCapacity=31;
          this->iLen=sprintf(this->lpBuffer,"%Iu",iNum);
       }

       return *this;
      }


      String& String::operator=(ssize_t iNum)
      {
       if(this->iCapacity>=32)
          this->iLen=sprintf(this->lpBuffer,"%Id",iNum);
       else
       {
          delete [] this->lpBuffer;
          this->lpBuffer=new char[32];
          this->iCapacity=31;
          this->iLen=sprintf(this->lpBuffer,"%Id",iNum);
       }

       return *this;
      }
   #endif


   String& String::operator=(int iNum)
   {
    if(this->iCapacity>=15)
       this->iLen=sprintf(this->lpBuffer,"%d",iNum);
    else
    {
       delete [] this->lpBuffer;
       this->lpBuffer=new char[16];
       this->iCapacity=15;
       this->iLen=sprintf(this->lpBuffer,"%d",iNum);
    }

    return *this;
   }


   String& String::operator=(unsigned int iNum)
   {
    if(this->iCapacity>=15)
       this->iLen=sprintf(this->lpBuffer,"%u",iNum);
    else
    {
       delete [] this->lpBuffer;
       this->lpBuffer=new char[16];
       this->iCapacity=15;
       this->iLen=sprintf(this->lpBuffer,"%u",iNum);
    }

    return *this;
   }


   String& String::operator=(double dblNumber)
   {
    if(this->iCapacity>=31)
       this->iLen=pSPrtf(this->lpBuffer,"%16.8f",dblNumber);
    else
    {
       delete [] this->lpBuffer;
       this->lpBuffer=new char[32];
       this->iLen=pSPrtf(this->lpBuffer,"%16.8f",dblNumber);
       this->iCapacity=31;
    }

    return *this;
   }
#endif


String String::operator+(const char ch)
{
 int iNewLen=this->iLen+1;

 String s(iNewLen,false);
 strcpy(s.lpBuffer,this->lpBuffer);
 s.lpBuffer[iNewLen-1]=ch;
 s.lpBuffer[iNewLen]='\0';
 s.iLen=iNewLen;

 return s;
}


String String::operator+(const char* pStr)
{
 int iNewLen=strlen(pStr)+this->iLen;
 String s(iNewLen,false);
 strcpy(s.lpBuffer,this->lpBuffer);
 strcat(s.lpBuffer,pStr);
 s.iLen=iNewLen;

 return s;
}


String String::operator+(String& strRef)
{
 int iNewLen=strRef.iLen+this->iLen;
 String s(iNewLen,false);
 strcpy(s.lpBuffer,this->lpBuffer);
 strcat(s.lpBuffer,strRef.lpBuffer);
 s.iLen=iNewLen;

 return s;
}


bool String::operator==(String& strRef)
{
 if(strcmp(this->lpStr(),strRef.lpStr())==0)
    return true;
 else
    return false;
}


bool String::operator==(const char* pStr)
{
 if(strcmp(this->lpStr(),pStr)==0)
    return true;
 else
    return false;
}


bool String::operator!=(char* pStr)
{
 if(strcmp(this->lpStr(),(char*)pStr)==0)
    return false;
 else
    return true;
}


void String::LTrim()
{
 size_t iCt=0;

 for(size_t i=0; i<this->iLen; i++)
 {
     if(this->lpBuffer[i]==9||this->lpBuffer[i]==10||this->lpBuffer[i]==13||this->lpBuffer[i]==32)
        iCt++;
     else
        break;
 }
 if(iCt)
 {
    for(size_t i=iCt; i<=this->iLen; i++)
        this->lpBuffer[i-iCt]=this->lpBuffer[i];
 }
 this->iLen=this->iLen-iCt;
}


void String::RTrim()
{
 int iCt=0;

 for(int i=this->iLen-1; i>0; i--)
 {
     if(this->lpBuffer[i]==9||this->lpBuffer[i]==10||this->lpBuffer[i]==13||this->lpBuffer[i]==32)
        iCt++;
     else
        break;
 }
 this->lpBuffer[this->iLen-iCt]=0;
 this->iLen=this->iLen-iCt;
}


void String::Trim()
{
 this->LTrim();
 this->RTrim();
}


String String::Left(size_t iNum)   //  strncpy = _tcsncpy
{
 if(iNum<this->iLen)
 {
    size_t iNewSize=(iNum*EXPANSION_FACTOR/16+1)*16;
    String sr(iNewSize,false);
    strncpy(sr.lpBuffer,this->lpBuffer,iNum);
    sr.lpBuffer[iNum]=0;
    sr.iLen=iNum;
    return sr;
 }
 else
 {
    String sr=*this;
    sr.iLen=this->iLen;
    return sr;
 }
}


String String::Right(size_t iNum)  //Returns Right$(strMain,iNum)
{
 if(iNum<this->iLen)
 {
    size_t iNewSize=(iNum*EXPANSION_FACTOR/16+1)*16;
    String sr(iNewSize,false);
    strncpy(sr.lpBuffer,this->lpBuffer+this->iLen-iNum,iNum);
    sr.lpBuffer[iNum]='\0';
    sr.iLen=iNum;
    return sr;
 }
 else
 {
    String sr=*this;
    sr.iLen=this->iLen;
    return sr;
 }
}


String String::Mid(size_t iStart, size_t iCount)
{
 if(iStart<1)
 {
    String sr;
    return sr;
 }
 if(iCount+iStart>this->iLen)
    iCount=this->iLen-iStart+1;
 String sr(iCount,false);
 strncpy(sr.lpBuffer,this->lpBuffer+iStart-1,iCount);
 sr.lpBuffer[iCount]='\0';
 sr.iLen=iCount;

 return sr;
}


int iMatch(char* pThis, const char* pStr, bool blnCaseSensitive, bool blnStartBeginning, int i, int iParamLen)
{
 if(blnCaseSensitive)
 {
    if(strncmp(pThis+i,pStr,iParamLen)==0)   //_tcsncmp
       return i+1;
    else
       return 0;
 }
 else
 {
    if(_strnicmp(pThis+i,pStr,iParamLen)==0)  //__tcsnicmp
       return i+1;
    else
       return 0;
 }
}


int String::InStr(const char* pStr, bool blnCaseSensitive, bool blnStartBeginning)
{
 int i,iParamLen,iRange,iReturn;

 if(*pStr==0)
    return 0;
 iParamLen=strlen(pStr);
 iRange=this->iLen-iParamLen;
 if(blnStartBeginning)
 {
    if(iRange>=0)
    {
       for(i=0; i<=iRange; i++)
       {
           iReturn=iMatch(this->lpBuffer,pStr,blnCaseSensitive,blnStartBeginning,i,iParamLen);
           if(iReturn)
              return iReturn;
       }
    }
 }
 else
 {
    if(iRange>=0)
    {
       for(i=iRange; i>=0; i--)
       {
           iReturn=iMatch(this->lpBuffer,pStr,blnCaseSensitive,blnStartBeginning,i,iParamLen);
           if(iReturn)
              return iReturn;
       }
    }
 }

 return 0;
}


int String::InStr(const String& s, bool blnCaseSensitive, bool blnStartBeginning)
{
 int i,iParamLen,iRange,iReturn;

 if(s.iLen==0)
    return 0;
 iParamLen=s.iLen;
 iRange=this->iLen-iParamLen;
 if(blnStartBeginning)
 {
    if(iRange>=0)
    {
       for(i=0; i<=iRange; i++)
       {
           iReturn=iMatch(this->lpBuffer,s.lpBuffer,blnCaseSensitive,blnStartBeginning,i,iParamLen);
           if(iReturn)
              return iReturn;
       }
    }
 }
 else
 {
    if(iRange>=0)
    {
       for(i=iRange; i>=0; i--)
       {
           iReturn=iMatch(this->lpBuffer,s.lpBuffer,blnCaseSensitive,blnStartBeginning,i,iParamLen);
           if(iReturn)
              return iReturn;
       }
    }
 }

 return 0;
}


int String::ParseCount(const char c)   //returns one more than # of
{                                      //delimiters so it accurately
 int iCtr=0;                           //reflects # of strings delimited
 char* p;                              //by delimiter.

 p=this->lpBuffer;
 while(*p)
 {
   if(*p==c)
      iCtr++;
   p++;
 }

 return ++iCtr;
}


void String::Parse(String* pStr, char delimiter, size_t iParseCount)
{
 char* pBuffer=(char*)operator new(this->iLen+1);
 if(pBuffer)
 {
    char* p=pBuffer;
    char* c=this->lpBuffer;
    while(*c)
    {
       if(*c==delimiter)
          *p=0;
       else
          *p=*c;
       p++, c++;
    }
    *p=0, p=pBuffer;
    for(size_t i=0; i<iParseCount; i++)
    {
        pStr[i]=p;
        p=p+pStr[i].iLen+1;
    }
    operator delete(pBuffer);
 }
}


String String::Replace(char* pMatch, char* pNew)  //strncmp = _tcsncmp
{
 size_t i,iLenMatch,iLenNew,iCountMatches,iExtra,iExtraLengthNeeded,iAllocation,iCtr;
 iLenMatch=strlen(pMatch);
 iCountMatches=0, iAllocation=0, iCtr=0;
 iLenNew=strlen(pNew);
 if(iLenNew==0)
 {
    String sr=this->Remove(pMatch,true); //return
    return sr;
 }
 else
 {
    iExtra=iLenNew-iLenMatch;
    for(i=0; i<this->iLen; i++)
    {
        if(strncmp(lpBuffer+i,pMatch,iLenMatch)==0)
           iCountMatches++;  //Count how many match strings
    }
    iExtraLengthNeeded=iCountMatches*iExtra;
    iAllocation=this->iLen+iExtraLengthNeeded;
    String sr(iAllocation,false);
    for(i=0; i<this->iLen; i++)
    {
        if(strncmp(this->lpBuffer+i,pMatch,iLenMatch)==0)
        {
           strcpy(sr.lpBuffer+iCtr,pNew);
           iCtr+=iLenNew;
           i+=iLenMatch-1;
        }
        else
        {
           sr.lpBuffer[iCtr]=this->lpBuffer[i];
           iCtr++;
        }
        sr.lpBuffer[iCtr]='\0';
    }
    sr.iLen=iCtr;
    return sr;
 }
}


String String::Remove(const char* pStr)    // Individual character removal, i.e., if this
{                                          // was passed in as pStr...
 unsigned int i,j,iStrLen,iParamLen;       //
 char *pThis, *pThat, *p;                  // abcdefg
 bool blnFoundBadTCHAR;                    //
                                           // ...then this would be returned in another
 iStrLen=this->iLen;                       // string
 String sr((int)iStrLen,false);            //
 iParamLen=strlen(pStr);                   // aceg
 pThis=this->lpBuffer;                     //
 p=sr.lpStr();
 for(i=0; i<iStrLen; i++)
 {
     pThat=(char*)pStr;
     blnFoundBadTCHAR=false;
     for(j=0; j<iParamLen; j++)
     {
         if(*pThis==*pThat)
         {
            blnFoundBadTCHAR=true;
            break;
         }
         pThat++;
     }
     if(!blnFoundBadTCHAR)
     {
        *p=*pThis;
        p++;
        *p='\0';
     }
     pThis++;
 }
 sr.iLen=strlen(sr.lpStr());

 return sr;
}


String String::Remove(const char* pMatch, bool blnCaseSensitive)
{
 size_t i,iCountMatches=0,iCtr=0;

 size_t iLenMatch=strlen(pMatch);
 for(i=0; i<this->iLen; i++)
 {
     if(blnCaseSensitive)
     {
        if(strncmp(lpBuffer+i,pMatch,iLenMatch)==0)  //_tcsncmp
           iCountMatches++;
     }
     else
     {
        if(_strnicmp(lpBuffer+i,pMatch,iLenMatch)==0) //__tcsnicmp
           iCountMatches++;
     }
 }
 int iAllocation=this->iLen-(iCountMatches*iLenMatch);
 String sr(iAllocation,false);
 for(i=0; i<this->iLen; i++)
 {
     if(blnCaseSensitive)
     {
        if(strncmp(this->lpBuffer+i,pMatch,iLenMatch)==0)
           i+=iLenMatch-1;
        else
        {
           sr.lpBuffer[iCtr]=this->lpBuffer[i];
           iCtr++;
        }
        sr.lpBuffer[iCtr]='\0';
     }
     else
     {
        if(_strnicmp(this->lpBuffer+i,pMatch,iLenMatch)==0)
           i+=iLenMatch-1;
        else
        {
           sr.lpBuffer[iCtr]=this->lpBuffer[i];
           iCtr++;
        }
        sr.lpBuffer[iCtr]='\0';
     }
 }
 sr.iLen=iCtr;
 return sr;
}


#ifdef FORMAT_MONEY
   void String::Format(double dblNumber, size_t iFldLen, size_t iDecPlaces)
   {
    char szNumber[24], szFormat[8], szDecPlaces[4], szDecimal[16];
    size_t iLen=0,iDot=0;
    SSIZE_T iNumber=0;
    SSIZE_T iDiff=0;
    String strTmp;

    memset(szNumber,0,24*sizeof(char));
    memset(szFormat,0,8*sizeof(char));
    memset(szDecPlaces,0,4*sizeof(char));
    memset(szDecimal,0,16*sizeof(char));
    #ifdef x64
       pSPrtf(szDecPlaces,"%llu",iDecPlaces);
    #else
       pSPrtf(szDecPlaces,"%u",iDecPlaces);
    #endif
    strcpy(szFormat,(char*)"%0.");
    strcat(szFormat,szDecPlaces);
    strcat(szFormat,(char*)"f");
    iLen=pSPrtf(szNumber,szFormat,dblNumber);
    if(iDecPlaces)  // At Least 1 Digit To Right Of Decimal Point
    {
       iNumber=(SSIZE_T)dblNumber;
       for(size_t i=0; i<iLen; i++)
       {
           if(szNumber[i]=='.')
           {
              iDot=i;
              break;
           }
       }
       strncpy(szDecimal,szNumber+iDot+1,iDecPlaces);
       if(dblNumber>=0)
          strTmp.Format(iNumber,0,',',true);
       else
       {
          if(iNumber==0)
          {
             strTmp.Format(iNumber,0,',',true);
             strTmp=(char*)"-"+strTmp;
          }
          else
             strTmp.Format(iNumber,0,',',true);
       }
       strTmp=strTmp+(char*)".";
       strTmp=strTmp+szDecimal;
       iLen=strTmp.Len();
       if(this->iCapacity<iLen || this->iCapacity<iFldLen)  // Code To Reallocate If this->lpBuffer Isn't Big Enough
       {
          size_t iNewSize,iNewLen;
          delete [] this->lpBuffer;
          iNewLen=iLen+iFldLen;
          iNewSize = (iNewLen/16 + 1) * 16;
          this->lpBuffer=new char[iNewSize];
          this->iCapacity=iNewSize-1;
       }
       iDiff=iFldLen-iLen;
       if(iDiff>0)
       {
          for(size_t i=0; i<(size_t)iDiff; i++)
              this->lpBuffer[i]=' ';
          this->lpBuffer[iDiff]=NULL;
       }
       else
          this->lpBuffer[0]=NULL;
       strcat(this->lpBuffer,strTmp.lpStr());
       this->iLen=strlen(this->lpBuffer);
    }
    else // No Decimal Places
    {    // pAtoi64=(ssize_t (*)(const char*))GetProcAddress(hLib,"_atoi64");
       #ifdef x64
          strTmp.Format((SSIZE_T)pAtoi64(szNumber),0,',',true);
       #else
          strTmp.Format((SSIZE_T)atoi(szNumber),0,',',true);
       #endif
       iLen=strTmp.Len();
       if(this->iCapacity<iLen || this->iCapacity<iFldLen)  // Code To Reallocate If this->lpBuffer Isn't Big Enough
       {
          size_t iNewSize,iNewLen;
          delete [] this->lpBuffer;
          iNewLen=iLen+iFldLen;
          iNewSize = (iNewLen/16 + 1) * 16;
          this->lpBuffer=new char[iNewSize];
          this->iCapacity=iNewSize-1;
       }
       iDiff=iFldLen-iLen;
       if(iDiff>0)
       {
          for(size_t i=0; i<(size_t)iDiff; i++)
              this->lpBuffer[i]=' ';
          this->lpBuffer[iDiff]=NULL;
       }
       else
          this->lpBuffer[0]=NULL;
       strcat(this->lpBuffer,strTmp.lpStr());
    }
    this->iLen=strlen(this->lpBuffer);
   }


   void String::Format(double dblNum, int iFldWth, int iDecimal, bool blnRightJustified)  // This one doesn't insert commas
   {                                                                                      // every three digits
    if(iFldWth>0 && iFldWth<32)
    {
       if(iDecimal>=0 && iDecimal<19)
       {
          if(this->iCapacity<32)
          {
             delete [] this->lpBuffer;
             lpBuffer=new char[32];
             this->iCapacity=31;
          }
          String sFormat;
          if(blnRightJustified)
             sFormat=(char*)"%";
          else
             sFormat=(char*)"%-";
          String strFldWth(iFldWth);
          String strDecimalPoints(iDecimal);
          sFormat=sFormat+strFldWth+(char*)"."+strDecimalPoints+(char*)"f";
          this->iLen=pSPrtf(this->lpBuffer,sFormat.lpStr(),dblNum);  // either of these work
       }
    }
   }


   void String::Format(size_t iNumber, int iFldLen, char cSeperator, bool blnRightJustified)
   {
    char szBuf1[24];
    char szBuf2[24];
    size_t iDigit=0;
    size_t iLen=0;
    size_t iCtr=1;
    size_t j=0;

    memset(szBuf1,0,24*sizeof(char));
    memset(szBuf2,0,24*sizeof(char));
    #ifdef x64
       pSPrtf(szBuf1,"%Iu",iNumber);
    #else
       pSPrtf(szBuf1,"%Iu",iNumber);
    #endif
    _strrev(szBuf1);
    iLen=strlen(szBuf1);
    for(size_t i=0; i<iLen; i++)
    {
        if(iCtr==3)
        {
           iDigit++;
           szBuf2[j]=szBuf1[i];
           if(iDigit<iLen)
           {
              j++;
              szBuf2[j]=cSeperator;
           }
           j++, iCtr=1;
        }
        else
        {
           iDigit++;
           szBuf2[j]=szBuf1[i];
           j++, iCtr++;
        }
    }
    _strrev(szBuf2);                    // Done With Creating String With Commas
    memset(szBuf1,0,24*sizeof(char));   // Reuse szBuf1
    strcpy(szBuf1,szBuf2);              // szBuf1 Now Contains Comma Delimited Positive Value, but No Field Paddings Or R/L Justification
    size_t iRequiredBytes;              // Find out which of two is larger - length of string necessary
    iLen=strlen(szBuf1);                // or iFldLen
    if(iFldLen>(int)iLen)
       iRequiredBytes=iFldLen;
    else
       iRequiredBytes=iLen;
    if(iRequiredBytes>(size_t)this->iCapacity)
    {
       delete [] this->lpBuffer;
       int iNewSize=(iRequiredBytes*EXPANSION_FACTOR/16+1)*16;
       this->lpBuffer=new char[iNewSize];
       this->iCapacity=iNewSize-1;
    }
    memset(this->lpBuffer,0,this->iCapacity*sizeof(char));  // Now, at this point, szBuf1 Is Holding reformatted value, whether plus or minus, But no leading or trailing spaces.  And this->lpBuffer is large enough.
    SSIZE_T iDifference=iFldLen-iLen;                       // space = 32;  $ = 36;  , = 44
    if(blnRightJustified)
    {
       if(iDifference > 0)
       {
          for(size_t i=0; i<(size_t)iDifference; i++)
              this->lpBuffer[i]=' ';  // 32
       }
       strcat(this->lpBuffer,szBuf1);
    }
    else
    {
       strcpy(this->lpBuffer,szBuf1);
       if(iDifference>0)
       {
          for(size_t i=iLen; i<iDifference+iLen; i++)
              this->lpBuffer[i]=' ';  // 32
       }
    }
    this->iLen=strlen(this->lpBuffer);
   }


   void String::Format(ssize_t iNumber, int iFldLen, char cSeperator, bool blnRightJustified)
   {
    bool blnPositive;
    char szBuf1[24];
    char szBuf2[24];
    size_t iDigit=0;
    size_t iLen=0;
    size_t iCtr=1;
    size_t j=0;

    memset(szBuf1,0,24*sizeof(char));
    memset(szBuf2,0,24*sizeof(char));
    if(iNumber<0)
       blnPositive=false;
    else
       blnPositive=true;
    #ifdef x64
       iNumber=pAbs64(iNumber);
    #else
       iNumber=pAbs(iNumber);
    #endif
    #ifdef x64
       pSPrtf(szBuf1,"%Iu",(size_t)iNumber);
    #else
       pSPrtf(szBuf1,"%u",(size_t)iNumber);
    #endif
    _strrev(szBuf1);
    iLen=strlen(szBuf1);
    for(size_t i=0; i<iLen; i++)
    {
        if(iCtr==3)
        {
           iDigit++;
           szBuf2[j]=szBuf1[i];
           if(iDigit<iLen)
           {
              j++;
              szBuf2[j]=cSeperator;
           }
           j++, iCtr=1;
        }
        else
        {
           iDigit++;
           szBuf2[j]=szBuf1[i];
           j++, iCtr++;
        }
    }
    _strrev(szBuf2); // Done With Creating String With Commas

    memset(szBuf1,0,24*sizeof(char));  // Reuse szBuf1
    if(blnPositive)
       strcpy(szBuf1,szBuf2);
    else
    {
       szBuf1[0]='-';
       strcat(szBuf1,szBuf2);
    }   // szBuf1 Now Contains Comma Delimited Positive Or Negative Value, but No Field Paddings Or R/L Justification

    size_t iRequiredBytes;         // Find out which of two is larger - length of string necessary
    iLen=strlen(szBuf1);           // or iFldLen
    if(iFldLen>(int)iLen)
       iRequiredBytes=iFldLen;
    else
       iRequiredBytes=iLen;

    if(iRequiredBytes>(size_t)this->iCapacity)
    {
       delete [] this->lpBuffer;
       int iNewSize=(iRequiredBytes*EXPANSION_FACTOR/16+1)*16;
       this->lpBuffer=new char[iNewSize];
       this->iCapacity=iNewSize-1;
    }
    memset(this->lpBuffer,0,this->iCapacity*sizeof(char));   // Now, at this point, szBuf1 Is Holding reformatted value, whether plus or minus,
    SSIZE_T iDifference=iFldLen-iLen; // space = 32;  $ = 36;  , = 44   But no leading or trailing spaces.  And this->lpBuffer is large enough.
    if(blnRightJustified)
    {
       if(iDifference > 0)
       {
          for(size_t i=0; i<(size_t)iDifference; i++)
              this->lpBuffer[i]=' ';  // 32
       }
       strcat(this->lpBuffer,szBuf1);
    }
    else
    {
       strcpy(this->lpBuffer,szBuf1);
       if(iDifference>0)
       {
          for(size_t i=iLen; i<iDifference+iLen; i++)
              this->lpBuffer[i]=' ';  // 32
       }
    }
    this->iLen=strlen(this->lpBuffer);
   }


   void String::Money(double dblAmount, size_t iFldLen, bool blnDollarSign)
   {
    size_t iLen,iCharactersNeeded,iDifference;
    String strAmount;

    strAmount.Format(dblAmount,0,2);
    if(blnDollarSign)
       strAmount=(char*)"$"+strAmount;
    iLen=strAmount.iLen;
    if(iLen>iFldLen)
       iCharactersNeeded=iLen;
    else
       iCharactersNeeded=iFldLen;
    if(this->iCapacity<iCharactersNeeded)
    {
       delete [] this->lpBuffer;
       size_t iNewSize = (iCharactersNeeded/16 + 1) * 16;
       this->lpBuffer=new char[iNewSize];
       this->iCapacity=iNewSize-1;
    }
    if(iLen>iFldLen)
       iDifference=0;
    else
       iDifference=iFldLen-iLen;
    for(size_t i=0; i<iDifference; i++)
        this->lpBuffer[i]=32;
    this->lpBuffer[iDifference]='\0';
    strcat(this->lpBuffer,strAmount.lpStr());
    this->iLen=strlen(lpBuffer);
   }


   void String::Money(double dblAmount, size_t iFldLen, size_t iOneBasedOffsetDollarSign)
   {
    size_t iLen,iCharactersNeeded,iDifference;
    String strAmount;

    strAmount.Money(dblAmount,0,false);
    iLen=strAmount.iLen;
    if(iLen>iFldLen)
       iCharactersNeeded=iLen;
    else
       iCharactersNeeded=iFldLen;
    if(this->iCapacity<iCharactersNeeded)
    {
       delete [] this->lpBuffer;
       size_t iNewSize = (iCharactersNeeded/16 + 1) * 16;
       this->lpBuffer=new char[iNewSize];
       this->iCapacity=iNewSize-1;
    }
    if(iLen>iFldLen)
       iDifference=0;
    else
       iDifference=iFldLen-iLen;
    for(size_t i=0; i<iDifference; i++)
        this->lpBuffer[i]=32;
    this->lpBuffer[iDifference]='\0';
    strcat(this->lpBuffer,strAmount.lpStr());
    if(iOneBasedOffsetDollarSign && iOneBasedOffsetDollarSign<=iFldLen)
       this->lpBuffer[iOneBasedOffsetDollarSign-1]='$';
    this->iLen=strlen(lpBuffer);
   }
#endif


void String::Print(bool blnCrLf)
{
 printf("%s",lpBuffer);
 if(blnCrLf)
    printf("\n");
}


void String::Print(const char* pStr, bool blnCrLf)
{
 printf("%s%s",pStr,lpBuffer);
 if(blnCrLf)
    printf("\n");
}


char* String::lpStr()  
{
 return lpBuffer;
}


int String::Len(void)
{
 return this->iLen;
}


int String::Capacity(void)
{
 return this->iCapacity;
}


String::~String()   //String Destructor
{
 //printf("Entering Destructor!\n");
 //printf("  this = %p\n",this);
 //operator delete(lpBuffer);
 delete [] this->lpBuffer;
 lpBuffer=0;
 //printf("Leaving Destructor\n\n");
}   
