//=====================================================================================
//               Developed As An Addition To Matt Pietrek's LibCTiny.lib
//                            By Fred Harris, January 2016
//
//                    cl _strnicmp.cpp /c /W3 /DWIN32_LEAN_AND_MEAN 
//=====================================================================================
#include <windows.h>
#include <string.h>
#include <malloc.h>

extern "C" int __cdecl _strnicmp(const char* str1, const char* str2, size_t count)
{
 size_t iLen1=strlen(str1);
 size_t iLen2=strlen(str2);
 if(count>iLen1)
    return -1;
 if(count>iLen2)
    return 1;
 char* pStr1=(char*)malloc(count+1);
 char* pStr2=(char*)malloc(count+1);
 strncpy(pStr1,str1,count);
 strncpy(pStr2,str2,count);
 pStr1[count]=0;
 pStr2[count]=0;
 int iReturn=lstrcmpi(pStr1,pStr2);
 free(pStr1);
 free(pStr2);

 return iReturn;
}


/*
_CRT_SECURE_NO_WARNINGS
*/




/*
_strnicmp

Compare characters of two strings without regard to case.

int _strnicmp( const char *string1, const char *string2, size_t count );

Return Value

The return value indicates the relationship between the substrings as follows.

Return Value Description 
========================================================
< 0     string1 substring less than string2 substring 
0       string1 substring identical to string2 substring 
> 0     string1 substring greater than string2 substring 


Parameters

string1, string2    Null-terminated strings to compare
count               Number of characters to compare

Remarks

The _strnicmp function lexicographically compares, at most, the first count characters 
of string1 and string2. The comparison is performed without regard to case; _strnicmp 
is a case-insensitive version of strncmp. The comparison ends if a terminating null 
character is reached in either string before count characters are compared. If the 
strings are equal when a terminating null character is reached in either string before 
count characters are compared, the shorter string is lesser.

=======================================================================================

lstrcmpi

The lstrcmpi function compares two character strings. The comparison is not case sensitive. 

int lstrcmpi
(
  LPCTSTR lpString1,  // pointer to first string
  LPCTSTR lpString2   // pointer to second string
);
 
Parameters

lpString1    Pointer to the first null-terminated string to be compared. 
lpString2    Pointer to the second null-terminated string to be compared.
 
Return Values

If the string pointed to by lpString1 is less than the string pointed to by lpString2, the 
return value is negative. If the string pointed to by lpString1 is greater than the string 
pointed to by lpString2, the return value is positive. If the strings are equal, the return 
value is zero. 

Remarks

The lstrcmpi function compares two strings by checking the first characters against each 
other, the second characters against each other, and so on until it finds an inequality or 
reaches the ends of the strings. 

The function returns the difference of the values of the first unequal characters it 
encounters. For example, lstrcmpi determines that "abcz" is greater than "abcdefg" and 
returns the difference of z and d. 

The language (locale) selected by the user at setup time, or by using the control panel, 
determines which string is greater (or whether the strings are the same). If no language 
(locale) is selected, the system performs the comparison by using default values. 

For some locales, the lstrcmpi function may be insufficient. If this occurs, use 
CompareString to ensure proper comparison. For example, in Japan call CompareString with 
the IGNORE_CASE, IGNORE_KANATYPE, and IGNORE_WIDTH values to achieve the most appropriate 
non-exact string comparison. The IGNORE_KANATYPE and IGNORE_WIDTH values are ignored in 
non-Asian locales, so you can set these values for all locales and be guaranteed to have 
a culturally correct "insensitive" sorting regardless of the locale. Note that specifying 
these values slows performance, so use them only when necessary.
*/