//=====================================================================================
//               Developed As An Addition To Matt Pietrek's LibCTiny.lib
//                            By Fred Harris, January 2016
//
//                     cl strcmp.cpp /c /W3 /DWIN32_LEAN_AND_MEAN
//=====================================================================================
#include <windows.h>

extern "C" int __cdecl strcmp(const char* string1, const char* string2)   
{
 return lstrcmp(string1,string2);
}



/*
lstrcmp Function

Compares two character strings. The comparison is case-sensitive.

To perform a comparison that is not case-sensitive, use the lstrcmpi function.


Syntax

int lstrcmp(LPCTSTR lpString1, LPCTSTR lpString2);

Parameters

lpString1    [in] Pointer to the first null-terminated string to be compared. 
lpString2    [in] Pointer to the second null-terminated string to be compared.
 
Return Value

If the string pointed to by lpString1 is less than the string pointed to by lpString2, 
the return value is negative. If the string pointed to by lpString1 is greater than 
the string pointed to by lpString2, the return value is positive. If the strings are 
equal, the return value is zero. 

======================================================================================

int strcmp(const char* string1, const char* string2);

Parameters     string1, string2  Null-terminated strings to compare.
Libraries      All versions of the C run-time libraries.
Return Values  The return value for each of these functions indicates the lexicographic relation of string1 to string2.

               Value Relationship of string1 to string2
 
               < 0 string1 less than string2 
               0 string1 identical to string2 
               > 0 string1 greater than string2 

Remarks        The strcmp function compares string1 and string2 lexicographically and returns a value indicating their 
               relationship. wcscmp is the wide-character version of strcmp. The arguments and return value of wcscmp 
               are wide-character strings. These two functions behave identically otherwise.

=======================================================================================================================
*/
